package index.alg;

import index.entity.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class BuildInitialOccupy {

    public OccupyPoint build(ArrayList<LineSegment> linesList, WholeIndex wholeIndex) throws Exception{

        Double minX = wholeIndex.getMinX();
        Double minY = wholeIndex.getMinY();
        Integer numOfGapsX = wholeIndex.getNumOfGapsX();
        Integer numOfGapsY = wholeIndex.getNumOfGapsY();
        Double xGap =wholeIndex.getxGap();
        Double yGap = wholeIndex.getyGap();

        //占用情况
        ArrayList<ArrayList<BeginEndPoint>> occupyPoints = new ArrayList<>();
        CurveTool curveTool = new CurveTool();
        for (int i = 0 ; i < numOfGapsX;i++){
            //当前列的x，取网格的中部
            Double xTemp = minX + i * xGap + 0.5 * xGap;
            //与当前列相交的线段，应该为偶数
            ArrayList<LineSegment> lineSegments = findLine(xTemp,linesList);

            if (lineSegments.size() % 2 !=0){

                throw new Exception("初始图形不规范!");

            }

            Integer lineSegmentsSize = lineSegments.size();
            Integer halfLineSegmentSize = lineSegmentsSize / 2;

            //当前列的占用情况
            ArrayList<BeginEndPoint> beginEndPoints = new ArrayList<>();

            for (int j = 0;j < lineSegmentsSize - 1;j=j+2){

                LineSegment lineSegment1 = lineSegments.get(j);
                LineSegment lineSegment2 = lineSegments.get( j + 1);

                ArrayList<Double> paramList1 = lineSegment1.getLineParamList();
                ArrayList<Double> paramList2 = lineSegment2.getLineParamList();

                Double y1 = curveTool.getLineYValue(xTemp,paramList1);
                Double y2 = curveTool.getLineYValue(xTemp,paramList2);

                Point point1 = new Point(xTemp,y1);
                Point point2 = new Point(xTemp,y2);
                BeginEndPoint beginEndPoint = new BeginEndPoint(point1,point2);
                beginEndPoints.add(beginEndPoint);
            }

            occupyPoints.add(beginEndPoints);
        }

        OccupyPoint occupyPoint = new OccupyPoint();

        occupyPoint.setOccupyPoints(occupyPoints);

        return occupyPoint;

    }

    //从小到大排序
    public ArrayList<LineSegment> findLine(Double x ,ArrayList<LineSegment> linesList){

        ArrayList<LineSegment> lineSegments = new ArrayList<>();

        for (int i = 0 ; i < linesList.size();i++){

            LineSegment lineSegment = linesList.get(i);
            BeginEndPoint beginEndPoint = lineSegment.getBeginEndPoint();
            Point pointBegin = beginEndPoint.getBeginPoint();
            Point pointEnd = beginEndPoint.getEndPoint();
            Double beginX = pointBegin.getColX();
            Double endX = pointEnd.getColX();
            if (x >= beginX && x <= endX){
                lineSegments.add(lineSegment);
            }
        }


        CurveTool curveTool = new CurveTool();
        ArrayList<Map> lineAndHeight = new ArrayList<>();
        for (int i = 0 ; i < lineSegments.size();i++){

            LineSegment lineSegment = lineSegments.get(i);
            ArrayList<Double> paramList = lineSegment.getLineParamList();
            Double height = curveTool.getLineYValue(x,paramList);
            Map map = new HashMap();
            map.put("HEIGHT",height);
            map.put("LINE",lineSegment);
            lineAndHeight.add(map);

        }

        for (int i = 0 ; i < lineAndHeight.size() - 1;i++){

            Map mapI = lineAndHeight.get(i);
            Double heightI = (Double) mapI.get("HEIGHT");
            for (int j = i + 1 ; j < lineAndHeight.size();j++){
                Map mapJ = lineAndHeight.get(j);
                Double heightJ = (Double) mapJ.get("HEIGHT");
                if (heightI >  heightJ){
                        Map mapTemp = mapI;
                        mapI = mapJ;
                        mapJ = mapTemp;
                        lineAndHeight.set(i,mapI);
                        lineAndHeight.set(j,mapJ);
                }
            }
        }

        ArrayList<LineSegment> lineSegmentsAfterSort = new ArrayList<>();
        for (int i = 0;i < lineAndHeight.size();i++){

            Map map = lineAndHeight.get(i);
            LineSegment lineSegment = (LineSegment) map.get("LINE");
            lineSegmentsAfterSort.add(lineSegment);
        }

        return lineSegmentsAfterSort;

    }

}


