package index.fit;

import org.apache.commons.math3.fitting.GaussianCurveFitter;
import org.apache.commons.math3.fitting.PolynomialCurveFitter;
import org.apache.commons.math3.fitting.WeightedObservedPoint;
import index.entity.Point;
import index.alg.CurveTool;
import org.apache.commons.math3.fitting.WeightedObservedPoints;
import org.omg.CORBA.ARG_IN;

import java.util.ArrayList;

public class Fit {

    public double[] fitByPoints(ArrayList<Point> pointArrayList){
        int fitDegree = 3;//几阶拟合
        PolynomialCurveFitter polynomialCurveFitter = PolynomialCurveFitter.create(fitDegree);
        ArrayList<WeightedObservedPoint> weightedObservedPoints = new ArrayList<>();
        for (int i = 0 ; i < pointArrayList.size();i++){
            Point point = pointArrayList.get(i);
            Double x = point.getColX();
            Double y = point.getColY();
            WeightedObservedPoint weightedObservedPoint = new WeightedObservedPoint(1,x,y);
            weightedObservedPoints.add(weightedObservedPoint);
        }
        return polynomialCurveFitter.fit(weightedObservedPoints);
    }

    //fitPointsNum:多少个点拟合
    //fillScale:拟合后起始点到终点之间需要补充的点的数量，为了拟合后画出的曲线能够更加光滑

    public ArrayList<Point> halfCircleFit(ArrayList<Point> pointArrayList,Integer fitPointsNum,Integer fillScale,Integer excludeModel){

        ArrayList<Point> pointArrayListAfterFilled = new ArrayList<>();
        Integer pointsListLength = pointArrayList.size();
        Integer roundNumber = pointsListLength / fitPointsNum;
        Integer remainNumber = pointsListLength - roundNumber * fitPointsNum;
        CurveTool curveTool = new CurveTool();

        for (int i = 0 ; i < roundNumber; i++){

            Integer jBegin = fitPointsNum * i;
            Integer jEnd = jBegin + fitPointsNum;
//            ArrayList<Point> subArrayListForFit = (ArrayList<Point>) pointArrayList.subList(jBegin,jEnd);
            ArrayList<Point> subArrayListForFit = getSubArrayList(pointArrayList,jBegin,jEnd);
            //
//            subArrayListForFit = excludePoints(subArrayListForFit,excludeModel);//过滤点
            //
            double[] curveParam = fitByPoints(subArrayListForFit);
            Point pointBegin = subArrayListForFit.get(0);
            Point pointEnd = subArrayListForFit.get(subArrayListForFit.size() - 1);
            Double xBegin = pointBegin.getColX();//子list的起点的横坐标
            Double xEnd = pointEnd.getColX();//子list的终点的横坐标
            Double indexLength = (xEnd - xBegin)/(fillScale + 1);

            for (int k = 0 ; k < (fillScale + 2);k++){
                Double xForCal = xBegin + k * indexLength;
                Double y = curveTool.getCurveValue(xForCal,curveParam);
                Point point = new Point();
                point.setColX(xForCal);
                point.setColY(y);
                pointArrayListAfterFilled.add(point);
            }

        }
        if (remainNumber != 0){

            Point pointLeftBegin = pointArrayList.get(roundNumber * fitPointsNum);
            Point pointLeftEnd = pointArrayList.get(pointArrayList.size() - 1);
//        ArrayList<Point> subArrayListForFitLeft = (ArrayList<Point>) pointArrayList.subList(roundNumber * fitPointsNum,pointArrayList.size());
            ArrayList<Point> subArrayListForFitLeft = getSubArrayList(pointArrayList,roundNumber * fitPointsNum,pointArrayList.size());
//            subArrayListForFitLeft = excludePoints(subArrayListForFitLeft,excludeModel);
            double[] curveParamLeft = fitByPoints(subArrayListForFitLeft);
            Double xLeftBegin = pointLeftBegin.getColX();
            Double xLeftEnd = pointLeftEnd.getColX();
            Double indexLeftLength = (xLeftEnd - xLeftBegin)/(fillScale + 1);
            for (int k = 0 ; k < (fillScale + 2);k++){
                Double xForCal = xLeftBegin + k * indexLeftLength;
                Double y = curveTool.getCurveValue(xForCal,curveParamLeft);
                Point point = new Point();
                point.setColX(xForCal);
                point.setColY(y);
                pointArrayListAfterFilled.add(point);
            }

        }


        return pointArrayListAfterFilled;

    }

    public ArrayList<Point> allCircleFit(ArrayList<Point> pointArrayList,Integer fitPointsNum,Integer fillScale){

        ArrayList<Point> pointsAfterFill = new ArrayList<>();
        Integer length = pointArrayList.size();

//        ArrayList<Point> subList1 = (ArrayList<Point>) pointArrayList.subList(0,length / 2);

        ArrayList<Point> subList1 = getSubArrayList(pointArrayList,0,length / 2);

//        ArrayList<Point> subList2 = (ArrayList<Point>) pointArrayList.subList(length / 2,length);

        ArrayList<Point> subList2 = getSubArrayList(pointArrayList,length / 2,length);
        ArrayList<Point> subPointsAfterFill1 = halfCircleFit(subList1,fitPointsNum,fillScale,0);
        ArrayList<Point> subPointsAfterFill2 = halfCircleFit(subList2,fitPointsNum,fillScale,1);
        pointsAfterFill.addAll(subPointsAfterFill1);
        pointsAfterFill.addAll(subPointsAfterFill2);
        pointsAfterFill.add(pointsAfterFill.get(0));
        return pointsAfterFill;
    }

    public ArrayList<Point> allCircleFitDefault(ArrayList<Point> pointArrayList){

        ArrayList<Point> pointsAfterFill = allCircleFit(pointArrayList,20,10);
        return pointsAfterFill;
    }

    public ArrayList<Point> getSubArrayList(ArrayList<Point> parentArrayList,Integer beginIndex,Integer endIndex){

        ArrayList<Point> subArrayList = new ArrayList<>();
        for (int i = beginIndex ; i < endIndex ; i++ ){

            Point pointTemp = parentArrayList.get(i);
            subArrayList.add(pointTemp);
        }

        return subArrayList;

    }

    public ArrayList<Point> excludePoints(ArrayList<Point> pointsListForExclude,Integer excludeModel){

        ArrayList<Point> pointsArraListAfterExclude = new ArrayList<>();
        Point pointBegin = pointsListForExclude.get(0);
        Point pointEnd = pointsListForExclude.get(pointsListForExclude.size() - 1);
        pointsArraListAfterExclude.add(pointBegin);

        for (int i = 1 ; i < pointsListForExclude.size() - 1;i++){

            Point lastPoint = pointsListForExclude.get(i - 1);
            Point nextPoint = pointsListForExclude.get(i + 1);
            Point thisPoint = pointsListForExclude.get(i);
            Double lastPointY = lastPoint.getColY();
            Double nextPointY = nextPoint.getColY();
            Double thisPointY = thisPoint.getColY();

            if (excludeModel == 0){

                if (!(thisPointY < nextPointY && thisPointY < lastPointY)){

                    pointsArraListAfterExclude.add(thisPoint);

                }
            }else if (excludeModel == 1){

                if (!(thisPointY > nextPointY && thisPointY > lastPointY)){

                    pointsArraListAfterExclude.add(thisPoint);

                }

            }




        }

        pointsArraListAfterExclude.add(pointEnd);

        return pointsArraListAfterExclude;

    }

    public ArrayList<Point> excludePoints(ArrayList<Point> pointsArrayList){

        Integer pointsLength = pointsArrayList.size();
        ArrayList<Point> pointArrayListAfterExclude = new ArrayList<>();
        ArrayList<Point> subpointsArrayList1 = getSubArrayList(pointsArrayList,0,pointsLength / 2);
        ArrayList<Point> subpointsArrayList2 = getSubArrayList(pointsArrayList,pointsLength / 2,pointsLength);
        pointArrayListAfterExclude.addAll(excludePoints(subpointsArrayList1,0));
        pointArrayListAfterExclude.addAll(excludePoints(subpointsArrayList2,1));
        pointArrayListAfterExclude.add(pointArrayListAfterExclude.get(pointArrayListAfterExclude.size() - 1));
        return pointArrayListAfterExclude;

    }

    public double[] gaussFit(ArrayList<Point> pointArrayList){

        WeightedObservedPoints guassWeightObservedPoint = new WeightedObservedPoints();
        for (int i = 0 ; i < pointArrayList.size();i++){
            Point point = pointArrayList.get(i);
            guassWeightObservedPoint.add(point.getColX(),point.getColY());

        }
        double[] parameters = GaussianCurveFitter.create().fit(guassWeightObservedPoint.toList());
        System.out.println("高斯拟合");
        for (int i = 0 ; i < parameters.length;i++){
            System.out.println(parameters[i]);
        }
        System.out.println("高斯拟合结束");
        return parameters;
    }


}
